/* $Id: debug.c,v 1.2 2000/03/18 00:11:05 leonard Exp $ */
/* Copyright David Leonard, 2000. All rights reserved. */
#include <stdio.h>
#include <string.h>
#include <unistd.h>
#include <err.h>
#include <sys/types.h>
#include <sys/syscall.h>
#include <sys/ioctl.h>
#include <xfs/xfs_message.h>
#include <xfs/xfs_pioctl.h>
#include "debug.h"

int debugflag = 1;

/* credentials */
char *
c2a(cred)
	struct xfs_cred *cred;
{
	static char res[256];
	snprintf(res, sizeof res, "{uid=%u pag=%x}", cred->uid, cred->pag);
	return res;
}

/* handle */
char *
h2a(handle)
	struct xfs_handle *handle;
{
	static char res[256];
	snprintf(res, sizeof res, "{%u %u %u %u}",
		handle->a, handle->b, handle->c, handle->d
	);
	return res;
}

/* cache_handle */
char  *
ch2a(ch)
	struct xfs_cache_handle *ch;
{
	return "?";	/* XXX */
}

/* attribute */
char *
a2a(attr)
	struct xfs_attr *attr;
{
	static char res[512];
	char *p = res, *e = res + sizeof res;

	p += snprintf(p, e - p, "{ ");
	if (XA_VALID_TYPE(attr))
		p += snprintf(p, e - p, "%s ", 
		    attr->xa_type == XFS_FILE_NON ? "vnon" :
		    attr->xa_type == XFS_FILE_REG ? "vreg" :
		    attr->xa_type == XFS_FILE_DIR ? "vdir" :
		    attr->xa_type == XFS_FILE_BLK ? "vblk" :
		    attr->xa_type == XFS_FILE_CHR ? "vchr" :
		    attr->xa_type == XFS_FILE_LNK ? "vlnk" :
		    attr->xa_type == XFS_FILE_SOCK ? "vsock" :
		    attr->xa_type == XFS_FILE_FIFO ? "vfifo" :
		    attr->xa_type == XFS_FILE_BAD ? "vbad" :	"v?");
	if (XA_VALID_MODE(attr))
		p += snprintf(p, e - p, "mod %o ", attr->xa_mode);
	if (XA_VALID_NLINK(attr))
		p += snprintf(p, e - p, "ln %d ", attr->xa_nlink);
	if (XA_VALID_SIZE(attr))
		p += snprintf(p, e - p, "sz %qu ", attr->xa_size);
	if (XA_VALID_UID(attr))
		p += snprintf(p, e - p, "u %d ", attr->xa_uid);
	if (XA_VALID_GID(attr))
		p += snprintf(p, e - p, "g %d ", attr->xa_gid);
	if (XA_VALID_FILEID(attr))
		p += snprintf(p, e - p, "id %d ", attr->xa_fileid);

	p += snprintf(p, e - p, "}");
		
	return res;
}

/* node */
char *
n2a(node)
	struct xfs_msg_node *node;
{
	static char res[512];

	snprintf(res, sizeof res, "{hnd %s tok %s attr %s}",
		h2a(&node->handle),
		t2a(node->tokens),
		a2a(&node->attr)
	);
	return res;
}

/* token */
char *
t2a(tok)
	u_int tok;
{
	static char res[128];
	int n = 0, i;
	static struct { u_int bit; char *nm; } bits[] = {
		{XFS_OPEN_NR,	"nr"},
		{XFS_OPEN_SR,	"sr"},
		{XFS_OPEN_NW,	"nw"},
		{XFS_OPEN_EW,	"ew"},
		{XFS_ATTR_R,	"ar"},
		{XFS_ATTR_W,	"aw"},
		{XFS_DATA_R,	"dr"},
		{XFS_DATA_W,	"dw"},
		{XFS_LOCK_R,	"lr"},
		{XFS_LOCK_W,	"lw"},
		{0,0}
	};

	res[0] = '\0';
	for (i = 0; bits[i].nm; i++)
		if (tok & bits[i].bit) {
			if (n++)
				strlcat(res, ",", sizeof res);
			strlcat(res, bits[i].nm, sizeof res);
		}
	if (!n) 
		strlcat(res, "0", sizeof res);
	return res;
}

/*
 * Enable debugging in the kernel filesystem.
 * Must have configured kernel with 'option XFS_DEBUG'
 * Debugging flags found in <xfs/xfs_debug.h>.
 */
void
kdebug(fd, flags)
	int fd;
	int32_t flags;
{
	int error;
	struct ViceIoctl vi;

	vi.in_size = sizeof flags;
	vi.in = (caddr_t)&flags;
	vi.out_size = 0;
	vi.out = NULL;

	error = syscall(SYS_xfspioctl, 
		AFSCALL_PIOCTL, 	/* operation */
		NULL,			/* a_pathP */
		VIOC_XFSDEBUG,		/* a_opcode */
		&vi,			/* a_paramsP */
		0			/* a_followSymlinks */
	);

	if (error != 0) 
		warn("kdebug");
}
