/* $Id: dir.c,v 1.3 2000/02/29 10:21:55 leonard Exp $ */
/* Copyright David Leonard, 2000. All rights reserved. */
#include <stdio.h>
#include <err.h>
#include <string.h>
#include <stdlib.h>
#include <unistd.h>
#include <dirent.h>

/* This macro comes from OpenBSD's <sys/dirent.h> */
#ifndef DIRENT_SIZE
#define DIRENT_SIZE(dp) \
    ((sizeof (struct dirent) - (MAXNAMLEN+1)) + (((dp)->d_namlen+1 + 3) &~ 3))
#endif

/*
 * Operations to build a directory
 */

extern int debugflag;

static char *zero;

int
dir_write(fd, de)
	int fd;
	struct dirent *de;
{
	int pagoff;
	int pad;
	size_t reclen;

	if (!zero)
		zero = calloc(DIRBLKSIZ, 1);

	pagoff = (int)(lseek(fd, 0, SEEK_CUR) % DIRBLKSIZ);
	reclen = DIRENT_SIZE(de);

	/* Conservatively pad if we can't fit another dirent after */
	if (pagoff + reclen > DIRBLKSIZ)
		errx(1, "dir_write: odd dirent %u+%u", pagoff, reclen);

	if (pagoff + reclen + sizeof *de > DIRBLKSIZ) 
		pad = DIRBLKSIZ - (pagoff + reclen);
	else
		pad = 0;

	if (debugflag)
		fprintf(stderr, ">> len=%u+%u %u %.*s\n", 
		    reclen, pad, de->d_fileno, de->d_namlen, de->d_name);

	de->d_reclen = reclen + pad;

	if (write(fd, de, reclen) == -1)
		return -1;
	if (pad && write(fd, zero, (size_t)pad) == -1)
		return -1;

	return 0;
}

void
dir_close(fd)
	int fd;
{
	int pagoff, pad;

	pagoff = (int)(lseek(fd, 0, SEEK_CUR) % DIRBLKSIZ);
	if (pagoff) {
		pad = DIRBLKSIZ - pagoff;
		if (debugflag)
			fprintf(stderr, ">> dir_close: padded %u\n", pad);
		write(fd, zero, (size_t)pad);
	}
	close(fd);
}
