/* $Id: ftppasswd.c,v 1.3 2000/02/29 04:15:48 leonard Exp $ */
/* Copyright David Leonard, 2000. All rights reserved. */
#include <stdio.h>
#include <stdlib.h>
#include <sys/stat.h>
#include <sys/dirent.h>
#include <errno.h>
#include <unistd.h>
#include <fcntl.h>
#include <xfs/xfs_message.h>
#include "inum.h"
#include "xfs.h"
#include "node.h"
#include "ftproot.h"

/*
 * ftppasswd file: s/ftppasswd/something/g
 *		  s/fpn/sn/g;
 * extern struct node_class ftppasswd_class;
 */

struct ftppasswd_node {
	struct node		node;
	struct node		*ftproot;
	int			isvalid;
};
extern struct node_class ftppasswd_class;

static int
ftppasswd_getattr(fpn)
	struct ftppasswd_node *fpn;
{
	return 0;
}

/* The password has been set. Try making a connection */
static int
ftppasswd_putdata(fpn)
	struct ftppasswd_node *fpn;
{

	fpn->isvalid = 1;
	ftproot_lock(fpn->ftproot);
	ftproot_connect(fpn->ftproot);
	ftproot_unlock(fpn->ftproot);
	return 0;
}

static void
ftppasswd_dealloc(fpn)
	struct ftppasswd_node *fpn;
{
	inum_reclaim(fpn->node.stat.st_ino);
	node_deinit(&fpn->node);
	free(fpn);
}

static void
ftppasswd_repr(fpn, f)
	struct ftppasswd_node *fpn;
	FILE *f;
{
	fprintf(f, "<ftppasswd>");
}

struct node_class ftppasswd_class = {
	ftppasswd_getattr,		/* getattr */
	(putattr_t)enoop,		/* putattr */
	(getdata_t)enoop,		/* getdata */
	ftppasswd_putdata,		/* putdata */
	(create_t)enotdir,		/* create  */
	(destroy_t)enotdir,		/* destroy */
	(lookup_t)enotdir,		/* lookup  */
	(rename_t)enotdir,		/* rename  */
	ftppasswd_dealloc,		/* dealloc */
	ftppasswd_repr,			/* repr    */
};

struct node *
ftppasswd_new(ftproot)
	struct node *ftproot;
{
	struct ftppasswd_node *fpn;

	fpn = (struct ftppasswd_node *)malloc(sizeof (struct ftppasswd_node));
	node_init(&fpn->node, &ftppasswd_class, ftproot->ctx);

	fpn->ftproot = ftproot;
	fpn->node.stat.st_gid = 0;
	fpn->node.stat.st_mode = S_IWUSR | S_IFREG;
	fpn->node.stat.st_ino = inum_new();

	return (struct node *)fpn;
}

int
ftppasswd_invalidate(n)
	struct node *n;
{
	struct ftppasswd_node *fpn = (struct ftppasswd_node *)n;
	int fd;

	/* Empty the password file */
	fpn->isvalid = 1;
	fd = node_open_file(&fpn->node, O_WRONLY | O_TRUNC);
	if (fd < 0)
		return -1;
	close(fd);
	fpn->node.stat.st_size = 0;
	node_installattr(&fpn->node);
	return 0;
}

int
ftppasswd_getpasswd(n, buf, sz)
	struct node *n;
	char *buf;
	size_t sz;
{
	struct ftppasswd_node *fpn = (struct ftppasswd_node *)n;
	int fd;
	int len;

	if (!fpn->isvalid) {
		errno = EINVAL;
		return -1;
	}

	/* Empty the password file */
	fd = node_open_file(&fpn->node, O_RDONLY);
	len = read(fd, buf, sz - 1);
	close(fd);
	if (len == -1)
		return -1;

	/* Drop trailing newline */
	if (len > 0 && buf[len - 1] == '\n')
		len--;

	buf[len] = '\0';
	return len;
}
