/* $Id: hash.c,v 1.2 2000/02/29 04:15:49 leonard Exp $ */
/* Copyright David Leonard, 2000. All rights reserved. */
#include <stdio.h>
#include <stdlib.h>
#include <fcntl.h>
#include <errno.h>
#include <string.h>
#include <db.h>
#include "hash.h"

/*
 * Use the Berkeley DB in-memory hashtable implementation.
 */

struct hash {
	DB	*db;
	int	first;
};

struct hash *
hash_new(void)
{
	struct hash *h;

	h = (struct hash *)malloc(sizeof (struct hash));
	if (h == NULL)
		return NULL;
	h->db = dbopen(NULL, O_RDWR, 0600, DB_HASH, NULL);
	if (h->db == NULL) {
		free(h);
		return NULL;
	}
	h->first = 1;
	return h;
}

int
hash_add(h, key, val)
	struct hash *h;
	const char *key;
	struct node *val;
{
	DBT k, v;

	k.data = (void *)key;
	k.size = strlen(key);
	v.data = &val;
	v.size = sizeof val;
	return (*h->db->put)(h->db, &k, &v, 0);
}

int
hash_del(h, key)
	struct hash *h;
	const char *key;
{
	DBT k;

	k.data = (void *)key;
	k.size = strlen(key);
	return (*h->db->del)(h->db, &k, 0);
}

void
hash_free(h)
	struct hash *h;
{

	(*h->db->close)(h->db);
	free(h);
}

int
hash_get(h, key, valp)
	struct hash *h;
	const char *key;
	struct node **valp;
{
	DBT k, v;
	int ret;

	k.data = (void *)key;
	k.size = strlen(key);
	v.data = NULL;
	v.size = 0;

	ret = (*h->db->get)(h->db, &k, &v, 0);
	if (ret == 0 && v.size == sizeof *valp)
		*valp = *(struct node **)v.data;
	else if (ret == 0) {
		errno = EINVAL;
		ret = -1;
	}
	return ret;
}


void
hash_first(h)
	struct hash *h;
{
	h->first = 1;
}

int
hash_next(h, key, keysz, valp)
	struct hash *h;
	char *key;
	size_t keysz;
	struct node **valp;
{
	DBT k, v;
	int ret;

	k.data = NULL;
	k.size = 0;
	v.data = NULL;
	v.size = 0;

	if (h->first) {
		h->first = 0;
		ret = (*h->db->seq)(h->db, &k, &v, R_FIRST);
	} else
		ret = (*h->db->seq)(h->db, &k, &v, R_NEXT);

	if (ret == 0 && v.size == sizeof *valp && keysz > k.size) {
		*valp = *(struct node **)v.data;
		memcpy(key, k.data, k.size);
		key[k.size] = '\0';
	} else if (ret == 0) {
		errno = EINVAL;
		ret = -1;
	}

	return ret;
}
