/* $Id: ihash.c,v 1.1 2000/02/29 04:16:08 leonard Exp $ */
/* Copyright David Leonard, 2000. All rights reserved. */
#include <stdio.h>
#include <stdlib.h>
#include <fcntl.h>
#include <errno.h>
#include <string.h>
#include <db.h>
#include "ihash.h"

/*
 * Use the Berkeley DB in-memory ihashtable implementation.
 */

struct ihash {
	DB	*db;
	int	first;
};

struct ihash *
ihash_new(void)
{
	struct ihash *h;

	h = (struct ihash *)malloc(sizeof (struct ihash));
	if (h == NULL)
		return NULL;
	h->db = dbopen(NULL, O_RDWR, 0600, DB_HASH, NULL);
	if (h->db == NULL) {
		free(h);
		return NULL;
	}
	h->first = 1;
	return h;
}

int
ihash_add(h, key, val)
	struct ihash *h;
	const char *key;
	ino_t val;
{
	DBT k, v;

	k.data = (void *)key;
	k.size = strlen(key);
	v.data = &val;
	v.size = sizeof val;
	return (*h->db->put)(h->db, &k, &v, 0);
}

int
ihash_del(h, key)
	struct ihash *h;
	const char *key;
{
	DBT k;

	k.data = (void *)key;
	k.size = strlen(key);
	return (*h->db->del)(h->db, &k, 0);
}

void
ihash_free(h)
	struct ihash *h;
{

	(*h->db->close)(h->db);
	free(h);
}

int
ihash_get(h, key, valp)
	struct ihash *h;
	const char *key;
	ino_t *valp;
{
	DBT k, v;
	int ret;

	k.data = (void *)key;
	k.size = strlen(key);
	v.data = NULL;
	v.size = 0;

	ret = (*h->db->get)(h->db, &k, &v, 0);
	if (ret == 0 && v.size == sizeof *valp)
		*valp = *(ino_t *)v.data;
	else if (ret == 0) {
		errno = EINVAL;
		ret = -1;
	}
	return ret;
}

