/* $Id: node.h,v 1.3 2000/02/29 04:15:49 leonard Exp $ */
/* Copyright David Leonard, 2000. All rights reserved. */

struct node_class;

struct ncred {
	uid_t uid;
};

struct node {
	struct node_class	*class;
	struct xfs_cache_handle	file;			/* data */
	int			file_valid;
	int			file_external;
	int			tokens;
	struct stat		stat;			/* attributes */
	struct xfs_ctx		*ctx;
#if 0
	struct node 		*parent;		/* XXX */
	char			name[MAXNAMLEN + 1];	/* XXX */
	int			namelen;		/* XXX */
#endif
};

typedef int		(*getattr_t)(struct node *);
typedef int		(*putattr_t)(struct node *, struct stat *);
typedef int		(*getdata_t)(struct node *);
typedef int		(*putdata_t)(struct node *);
typedef int		(*create_t)(struct node *, const char *, struct stat *);
typedef int		(*destroy_t)(struct node *, const char *);
typedef struct node *	(*lookup_t)(struct node *, const char *, 
				       struct ncred *);
typedef int		(*rename_t)(struct node *, const char *,
				  struct node *, const char *);
typedef void		(*dealloc_t)(struct node *);
typedef void		(*repr_t)(struct node *, FILE *);

int	node_getattr(struct node *);
int	node_putattr(struct node *, struct stat *);
int	node_getdata(struct node *);
int	node_putdata(struct node *);
int	node_create(struct node *, const char *, struct stat *);
int	node_destroy(struct node *, const char *);
struct node *	node_lookup(struct node *, const char *, struct ncred *);
int	node_rename(struct node *, const char *, struct node *, const char *);
void	node_dealloc(struct node *);

/* If a method returns NULL or -1, it should set errno appropriately */
struct node_class {
	getattr_t	getattr;	/* .stat needs filling */
	putattr_t	putattr;	/* .stat was modified */
	getdata_t	getdata;	/* .file needs filling */
	putdata_t	putdata;	/* .file was modified */

	create_t	create;		/* create child node */
	destroy_t	destroy;	/* destroy child node */
	lookup_t	lookup;		/* find child node */
	rename_t	rename;		/* re-parent child */

	dealloc_t	dealloc;	/* deallocte storage */
	repr_t		repr;		/* print description */
};

int	node_alloc_file(struct node *);
int	node_open_file(struct node *, int);
off_t	node_getsize_file(struct node *);
int	node_setsize_file(struct node *, off_t);
int	node_dealloc_file(struct node *);

struct	node * handle_to_node(const struct xfs_handle *);
void	node_to_handle(const struct node *, struct xfs_handle *);
void	node_init(struct node *, struct node_class *, struct xfs_ctx *);
void	node_deinit(struct node *);

int	eperm(struct node *);		/* Permission denied. */
int	eacces(struct node *);		/* Access denied */
int	enotdir(struct node *);		/* Not a directory */
int	enodev(struct node *);		/* Operation not supported */
int	enoop(struct node *);		/* No error */

#if 0 /* defined(__OPTIMIZE__) */
#define	node_getattr(n)		((*(n)->class->getattr)(n))
#define	node_putattr(n,s)	((*(n)->class->putattr)(n,s))
#define	node_getdata(n)		((*(n)->class->getdata)(n))
#define	node_putdata(n)		((*(n)->class->putdata)(n))
#define	node_create(n,nm,s)	((*(n)->class->create)(n,nm,s))
#define	node_destroy(n,nm)	((*(n)->class->destroy)(n,nm))
#define node_lookup(n,nm,c)	((*(n)->class->lookup)(n,nm,c))
#define	node_rename(n,f,m,t)	((*(n)->class->rename)(n,f,m,t))
#define	node_dealloc(n)		((*(n)->class->dealloc)(n))
#endif
#define	node_repr(n,f)						 \
	do { if ((n) == NULL) fprintf(f, "<null>");		 \
	     else if ((n)->class->repr) (*(n)->class->repr)(n,f);\
	     else fprintf(f, "<%p>", n); } while (0)

int node_installroot(struct node *root);
int node_installnode(struct node *parent, struct node *child, const char *name);
int node_installattr(struct node *node);
int node_installdata(struct node *node, int inval);
int node_invalidnode(struct node *node);
