/* $Id: xfs.h,v 1.1.1.1 2000/02/28 19:59:51 leonard Exp $ */
/* Copyright David Leonard, 2000. All rights reserved. */

#include <xfs/xfs_message.h>

struct node;
struct xfs_ctx;
struct xfs_opctx;

struct xfs_kernel_ops {
	int (*installroot)(struct xfs_ctx *, struct xfs_msg_node *);
	int (*installnode)(struct xfs_ctx *, xfs_handle *, const char *, 
		struct xfs_msg_node *);
	int (*installattr)(struct xfs_ctx *, struct xfs_msg_node *);
	int (*installdata)(struct xfs_ctx *, struct xfs_msg_node *,
		const char *, struct xfs_cache_handle *, u_int);
	int (*invalidnode)(struct xfs_ctx *, xfs_handle *);
	int (*updatefid)(struct xfs_ctx *, xfs_handle *, xfs_handle *);
};

struct xfs_user_ops {
	int (*version)(struct xfs_opctx *, void *);
	int (*getroot)(struct xfs_opctx *, xfs_cred *);
	int (*getnode)(struct xfs_opctx *, xfs_cred *, xfs_handle *, 
		const char *);
	int (*getattr)(struct xfs_opctx *, xfs_cred *, xfs_handle *);
	int (*getdata)(struct xfs_opctx *, xfs_cred *, xfs_handle *, 
		u_int);
	void (*inactivenode)(struct xfs_opctx *, xfs_handle *, u_int);
	int (*open)(struct xfs_opctx *, xfs_cred *, xfs_handle *, u_int);
	int (*putdata)(struct xfs_opctx *, xfs_handle *, struct xfs_attr *,
		xfs_cred *, u_int);
	int (*putattr)(struct xfs_opctx *, xfs_handle *, struct xfs_attr *,
		xfs_cred *);
	int (*create)(struct xfs_opctx *, xfs_handle *, const char *, 
		struct xfs_attr *, int, xfs_cred *);
	int (*mkdir)(struct xfs_opctx *, xfs_handle *, const char *,
		struct xfs_attr *, xfs_cred *);
	int (*link)(struct xfs_opctx *, xfs_handle *, const char *,
		xfs_handle *, xfs_cred *);
	int (*symlink)(struct xfs_opctx *, xfs_handle *, const char *, const char *,
		struct xfs_attr *, xfs_cred *);
	int (*remove)(struct xfs_opctx *, xfs_handle *, const char *,
		xfs_cred *);
	int (*rmdir)(struct xfs_opctx *, xfs_handle *, const char *,
		xfs_cred *);
	int (*rename)(struct xfs_opctx *, xfs_handle *, const char *,
		xfs_handle *, const char *, xfs_cred *);
	int (*pioctl)(struct xfs_opctx *, int, xfs_cred *, int, int,
		const char *, char *, xfs_handle *);
	int (*advlock)(struct xfs_opctx *, xfs_handle *, xfs_cred *,
		xfs_locktype_t, xfs_lockid_t);
};

struct xfs_ctx {
	int	x_fd;
	struct xfs_kernel_ops *x_kops;
	struct xfs_user_ops *x_uops;
	struct node *root;
};

#if 0
int xfs_installroot(struct xfs_opctx *, struct xfs_msg_node *);
int xfs_installnode(struct xfs_opctx *, xfs_handle *, const char *, 
		struct xfs_msg_node *);
int xfs_installattr(struct xfs_opctx *, struct xfs_msg_node *);
int xfs_installdata(struct xfs_opctx *, struct xfs_msg_node *,
		const char *, struct xfs_cache_handle *, u_int);
int xfs_invalidnode(struct xfs_opctx *, xfs_handle *);
int xfs_updatefid(struct xfs_opctx *, xfs_handle *, xfs_handle *);
#endif

struct xfs_ctx* xfs_new(struct xfs_user_ops *);
void xfs_mainloop(struct xfs_ctx *);

int  xfs_fhget(const char *, int, struct xfs_cache_handle *);
int  xfs_fhopen(const struct xfs_cache_handle *, int);
struct xfs_ctx * xfs_opctx_to_ctx(struct xfs_opctx *);


